package svc

import (
	"applet/app/e"
	"applet/app/enum"
	"applet/app/md"
	"applet/app/utils"
	db "code.fnuoos.com/zhimeng/model.git/src"
	"code.fnuoos.com/zhimeng/model.git/src/implement"
	model2 "code.fnuoos.com/zhimeng/model.git/src/model"
	"fmt"
	"github.com/gin-gonic/gin"
	"github.com/jinzhu/copier"
	"strings"
)

func DataCenterTable(c *gin.Context, req md.DataCenterTableReq) md.DataCenterTableRes {
	var req1 md.DataCenterRecordReq
	copier.Copy(&req1, &req)
	nativeString, _ := comm(c, 0, req1)
	list := make([]md.DataCenterTableData, 0)
	for _, v := range nativeString {
		tmp := md.DataCenterTableData{
			Date:          v["date"],
			ExposureCount: v["exposure_count"],
			MediaRevenue:  v["media_revenue"],
			Ecpm:          v["ecpm"],
		}
		list = append(list, tmp)
	}
	res := md.DataCenterTableRes{
		List: list,
	}
	return res
}
func DataCenterRecordTotal(c *gin.Context, req md.DataCenterTableReq) md.DataCenterTotalData {
	appId := GetAppletId(c, req.AppId, req.Platform)
	sql := `
		SELECT 
			SUM(exposure_count) as exposure_count,
			SUM(click_count) as click_count,
			SUM(click_rate) as click_rate,
			SUM(ecpm) as ecpm,
			SUM(media_revenue) as media_revenue
		FROM generate_wx_ad_data
		where %s 
`
	where := "uuid=" + c.GetString("mid")
	if req.AppId != "" || req.Platform != "" {
		where += " and app_id in(" + appId + ")"
	}
	if req.AdType != "" {
		where += " and ad_slot='" + req.AdType + "'"
	}
	if req.StartDate != "" {
		where += " and date>='" + req.StartDate + "'"
	}
	if req.EndDate != "" {
		where += " and date<='" + req.EndDate + "'"
	}
	sql = fmt.Sprintf(sql, where)
	nativeString, _ := db.QueryNativeString(db.Db, sql)
	res := md.DataCenterTotalData{}
	for _, v := range nativeString {
		res = md.DataCenterTotalData{
			Date:          "-",
			AppName:       "-",
			PlatformName:  "-",
			AdvName:       "-",
			ExposureCount: v["exposure_count"],
			MediaRevenue:  v["media_revenue"],
			Ecpm:          v["ecpm"],
			ClickRate:     v["click_rate"],
			ClickCount:    v["click_count"],
		}
	}
	return res
}
func DataCenterRecordList(c *gin.Context, req md.DataCenterRecordReq) md.DataCenterRecordRes {
	nativeString, total := comm(c, 1, req)
	list := make([]md.DataCenterTotalData, 0)
	for _, v := range nativeString {
		NewAppletApplicationDb := implement.NewAppletApplicationDb(MasterDb(c))
		app, _ := NewAppletApplicationDb.GetAppletApplicationListByAppid(v["app_id"])
		platform := ""
		name := ""
		if app != nil {
			name = app.Name
			platform = app.Platform
		}
		tmp := md.DataCenterTotalData{
			Date:          v["date"],
			AppName:       name,
			PlatformName:  md.AppletPlatformMap[platform],
			AdvName:       enum.AdunitType(v["ad_slot"]).String(),
			ExposureCount: v["exposure_count"],
			MediaRevenue:  v["media_revenue"],
			Ecpm:          v["ecpm"],
			ClickRate:     v["click_rate"],
			ClickCount:    v["click_count"],
		}
		list = append(list, tmp)
	}
	res := md.DataCenterRecordRes{
		List:  list,
		Total: total,
	}
	return res
}
func comm(c *gin.Context, isTotal int, req md.DataCenterRecordReq) ([]map[string]string, int64) {
	appId := GetAppletId(c, req.AppId, req.Platform)
	sql := `
		SELECT 
			%s
		FROM generate_wx_ad_data
		where %s %s
`
	where := "uuid=" + c.GetString("mid")
	if req.AppId != "" || req.Platform != "" {
		where += " and app_id in(" + appId + ")"
	}
	if req.AdType != "" {
		where += " and ad_slot='" + req.AdType + "'"
	}
	if req.StartDate != "" {
		where += " and date>='" + req.StartDate + "'"
	}
	if req.EndDate != "" {
		where += " and date<='" + req.EndDate + "'"
	}
	field := `*`
	start := (utils.StrToInt(req.Page) - 1) * utils.StrToInt(req.Limit)
	groupBy := ""
	if req.Page != "" {
		groupBy = " group by date,app_id,slot_id order by date desc,id desc limit " + utils.IntToStr(start) + "," + req.Limit
	} else {
		groupBy = " group by date order by date asc"
	}

	sql1 := fmt.Sprintf(sql, field, where, groupBy)
	nativeString, _ := db.QueryNativeString(db.Db, sql1)

	var total int64 = 0
	if isTotal == 1 {
		sqlTotal := fmt.Sprintf(sql, "COUNT(*) as count ", where)
		nativeStringTotal, _ := db.QueryNativeString(db.Db, sqlTotal)
		for _, v := range nativeStringTotal {
			total = utils.StrToInt64(v["count"])
		}
	}
	return nativeString, total
}
func DataCenterSelectData(c *gin.Context) {
	NewAppletApplicationDb := implement.NewAppletApplicationDb(MasterDb(c))
	appList, _ := NewAppletApplicationDb.FindAllAppletApplicationList()
	appMap := make(map[string][]map[string]interface{})
	for _, v := range appList {
		_, ok := appMap[v.Platform]
		if ok == false {
			appMap[v.Platform] = make([]map[string]interface{}, 0)
		}
		tmp := map[string]interface{}{
			"name":    v.Name,
			"app_id":  v.AppId,
			"ad_type": enum.AdTypeList,
		}
		appMap[v.Platform] = append(appMap[v.Platform], tmp)
	}
	platform := []map[string]interface{}{
		{
			"name":     "微信小程序",
			"platform": "wx_applet",
			"app_list": appMap["wx_applet"],
		},
	}
	e.OutSuc(c, platform, nil)
	return
}

// 应用
func GetAppletId(c *gin.Context, appId, platform string) string {
	mediumId := ""
	sess := MasterDb(c).Where("1=1")
	var ids = make([]string, 0)
	if appId != "" || platform != "" {
		ids = append(ids, "-1")
	}
	if platform != "" {
		var tmp []model2.AppletApplication
		if platform != "" {
			sess.And("platform = ? ", platform)
		}
		sess.Find(&tmp)
		for _, v := range tmp {
			ids = append(ids, utils.IntToStr(v.MediumId))
		}
	}
	if appId != "" {
		ids = []string{appId}
	}
	if appId != "" || platform != "" {
		mediumId = strings.Join(ids, ",")
	}
	return mediumId
}
